import os
import glob
import shutil
import pytest
import oommfc as oc
import discretisedfield as df
import pytest


@pytest.mark.oommf
def test_multiple_runs():
    name = "multiple_runs"

    # Remove any previous simulation directories.
    if os.path.exists(name):
        shutil.rmtree(name)

    L = 10e-9   # (m)
    cellsize = (5e-9, 5e-9, 5e-9)  # (m)
    mesh = oc.Mesh((0, 0, 0), (L, L, L), cellsize)

    system = oc.System(name=name)

    A = 1.3e-11  # (J/m)
    system.hamiltonian = oc.Exchange(A)

    gamma = 2.211e5  # (m/As)
    alpha = 0.02
    system.dynamics = oc.Precession(gamma) + oc.Damping(alpha)

    Ms = 8e5  # (A/m)
    system.m = df.Field(mesh, value=(1, 0.25, 0.1), norm=Ms)

    md = oc.MinDriver()
    md.drive(system)  # updates system.m in-place
    
    dirname = os.path.join(name, "drive-0")
    miffilename = os.path.join(dirname, "{}.mif".format(name))
    assert os.path.exists(dirname)
    assert os.path.isfile(miffilename)

    mif_file  = list(glob.iglob("{}/*.mif".format(dirname)))
    omf_files = list(glob.iglob("{}/*.omf".format(dirname)))
    odt_files = list(glob.iglob("{}/*.odt".format(dirname)))

    assert len(mif_file) == 1
    assert len(omf_files) > 1
    omffilename = os.path.join(dirname, "m0.omf")
    assert omffilename in omf_files

    assert len(odt_files) >= 1

    td = oc.TimeDriver()
    td.drive(system, t=100e-12, n=10)  # updates system.m in-place

    dirname = os.path.join(name, "drive-1")
    miffilename = os.path.join(dirname, "{}.mif".format(name))
    assert os.path.exists(dirname)
    assert os.path.isfile(miffilename)

    mif_file  = list(glob.iglob("{}/*.mif".format(dirname)))
    omf_files = list(glob.iglob("{}/*.omf".format(dirname)))
    odt_files = list(glob.iglob("{}/*.odt".format(dirname)))

    assert len(mif_file) == 1
    assert len(omf_files) == 11
    omffilename = os.path.join(dirname, "m0.omf")
    assert omffilename in omf_files

    assert len(odt_files) >= 1

    shutil.rmtree(name)



import os
import time
import pytest
import pkg_resources
import oommfc.oommf
from .hamiltonian import Exchange, UniaxialAnisotropy, \
    Demag, Zeeman, DMI, CubicAnisotropy, Hamiltonian
from .dynamics import Precession, Damping, STT, Dynamics
from .drivers import Driver, MinDriver, TimeDriver, HysteresisDriver
from .mesh import Mesh
from .system import System
from .data import Data
from micromagneticmodel.consts import mu0, e, me, kB, h, g, \
    hbar, gamma, muB, gamma0
from . import examples


__version__ = pkg_resources.get_distribution(__name__).version
__dependencies__ = pkg_resources.require(__name__)


def _run_tests(tag):
    args = ['-m', tag, '-v', '--pyargs', 'oommfc']  # pragma: no cover
    return pytest.main(args)  # pragma: no cover


def test():
    return(_run_tests('not travis and not docker'))  # pragma: no cover


def test_not_oommf():
    return(_run_tests('not oommf and not travis'))  # pragma: no cover


def test_oommf():
    return(_run_tests('oommf and not travis '
                      'and not docker'))  # pragma: no cover


def test_docker():
    return(_run_tests('docker'))  # pragma: no cover



import sys
import micromagneticmodel as mm


class DMI(mm.DMI):
    @property
    def _script(self):
        if self.crystalclass in ['t', 'o'] and sys.platform != 'win32':
            mif = '# DMI of crystallographic class T(O)\n'
            mif += 'Specify Oxs_DMI_T {\n'
        elif self.crystalclass == 'd2d' and sys.platform != 'win32':
            mif = '# DMI of crystallographic class D2d\n'
            mif += 'Specify Oxs_DMI_D2d {\n'
        elif self.crystalclass == 'cnv' and sys.platform != 'win32':
            mif = '# DMI of crystallographic class Cnv\n'
            mif += 'Specify Oxs_DMI_Cnv {\n'
        elif self.crystalclass == 'cnv' and sys.platform == 'win32':
            mif = '# DMI of crystallographic class Cnv\n'
            mif += 'Specify Oxs_DMExchange6Ngbr {\n'
        else:
            raise ValueError('The {} crystal class is not supported '
                             'on {} platform.'.format(self.crystalclass, sys.platform))

        mif += '  default_D {}\n'.format(self.D)
        mif += '  atlas :atlas\n'
        mif += '  D {\n'
        mif += '    atlas atlas {}\n'.format(self.D)
        mif += '  }\n'
        mif += '}\n\n'

        return mif



import sys
import pytest
import oommfc as oc
import micromagneticmodel.tests as mmt


def check_script(script, contains):
    assert script.count('\n') == 9
    assert script[0] == '#'
    assert script[-1] == '\n'

    for content in contains:
        assert content in script
            
class TestDMI(mmt.TestDMI):
    @pytest.mark.skipif(sys.platform == 'win32',
                        reason='Crystalographic class not supported on Windows.')
    def test_script_to(self):
        for D in self.valid_args:
            dmi = oc.DMI(D, crystalclass='t')
            check_script(dmi._script, contains=['Oxs_DMI_T'])

            dmi = oc.DMI(D, crystalclass='o')
            check_script(dmi._script, contains=['Oxs_DMI_T'])

    @pytest.mark.skipif(sys.platform == 'win32',
                        reason='Crystalographic class not supported on Windows.')
    def test_script_d2d(self):
        for D in self.valid_args:
            dmi = oc.DMI(D, crystalclass='d2d')
            check_script(dmi._script, contains=['Oxs_DMI_D2d'])

    @pytest.mark.skipif(sys.platform == 'win32',
                        reason='Crystalographic class not supported on Windows.')
    def test_script_cnv_linux_mac(self):
        for D in self.valid_args:
            dmi = oc.DMI(D, crystalclass='cnv')
            check_script(dmi._script, contains=['Oxs_DMI_Cnv'])

    @pytest.mark.skipif(sys.platform != 'win32',
                        reason='Crystalographic class not supported on Windows.')
    def test_script_cnv_win(self):
        for D in self.valid_args:
            dmi = oc.DMI(D, crystalclass='cnv')
            check_script(dmi._script, contains=['Oxs_DMExchange6Ngbr'])

    @pytest.mark.skipif(sys.platform != 'win32',
                        reason='Crystalographic class not supported on Windows.')
    def test_valueerror(self):
        with pytest.raises(ValueError):
            dmi = oc.DMI(D=1, crystalclass='t')
            dmi._script



import os
import glob
import json
import shutil
import datetime
import oommfc as oc
import oommfodt as oo
import discretisedfield as df
import micromagneticmodel as mm


class Driver(mm.Driver):
    def drive(self, system, overwrite=False, **kwargs):
        # This method is implemented in the derived class (TimeDriver,
        # MinDriver,...).
        self._checkargs(**kwargs)

        # Generate the necessary filenames.
        self.dirname = os.path.join(system.name,
                                    'drive-{}'.format(system.drive_number))
        self.omffilename = os.path.join(self.dirname, 'm0.omf')
        self.miffilename = os.path.join(self.dirname,
                                        '{}.mif'.format(system.name))
        self.jsonfilename = os.path.join(self.dirname, 'info.json')

        # Check whether a directory with the same name as system.name
        # already exists. If it does, warn the user and tell him that
        # he should pass overwrite=True to the drive method.
        self._checkdir(system, overwrite=overwrite)

        # Make a directory inside which OOMMF will be run.
        self._makedir()

        # Generate and save mif file.
        self._makemif(system, **kwargs)

        # Save system's initial magnetisation m0.omf file.
        self._makeomf(system)

        # Create json info file.
        self._makejson(system, **kwargs)

        # Run OOMMF.
        self._runoommf()

        # Update system's m and dt attributes if the derivation of E,
        # Heff, or energy density was not asked.
        if 'derive' not in kwargs:
            self._update_m(system)
            self._update_dt(system)

        # Increase the system's drive_number counter.
        system.drive_number += 1

    def _checkargs(self, **kwargs):
        raise NotImplementedError('This method is defined in a derived class')

    def _checkdir(self, system, overwrite=False):
        if os.path.exists(self.dirname):
            if not overwrite:
                msg = ('Directory with name={} already exists. If you want '
                       'to overwrite it, pass overwrite=True to the drive '
                       'method. Otherwise, change the name of the system '
                       'or delete the directory by running '
                       'system.delete().'.format(self.dirname))
                raise FileExistsError(msg)
            else:
                shutil.rmtree(system.name)

    def _makedir(self):
        if not os.path.exists(self.dirname):
            os.makedirs(self.dirname)

    def _makemif(self, system, **kwargs):
        mif = '# MIF 2.1\n\n'
        mif += system._script
        mif += self._script(system, **kwargs)

        with open(self.miffilename, 'w') as miffile:
            miffile.write(mif)

    def _makeomf(self, system):
        system.m.write(self.omffilename)

    def _makejson(self, system, **kwargs):
        info = {}
        info['drive_number'] = system.drive_number
        info['date'] = datetime.datetime.now().strftime('%Y-%m-%d')
        info['time'] = datetime.datetime.now().strftime('%H:%M:%S')
        info['driver'] = self.__class__.__name__
        info['args'] = kwargs

        with open(self.jsonfilename, 'w') as jsonfile:
            jsonfile.write(json.dumps(info))

    def _runoommf(self):
        oommf = oc.oommf.get_oommf_runner()
        oommf.call(argstr=self.miffilename)

    def _update_m(self, system):
        # An example .omf filename is:
        # test_sample-Oxs_TimeDriver-Magnetization-01-0000008.omf
        omffiles = glob.iglob(os.path.join(self.dirname,
                                           f'{system.name}*.omf'))
        lastomffile = sorted(omffiles)[-1]
        m_field = df.read(lastomffile)

        # This line exists because the mesh generated in df.read
        # method comes from the discrtisedfield module where the
        # _script method is not implemented.
        m_field.mesh = system.m.mesh

        system.m = m_field

    def _update_dt(self, system):
        system.dt = oo.read(os.path.join(self.dirname,
                                         f'{system.name}.odt'))



import os
import glob
import shutil
import pytest
import oommfc as oc
from .test_driver import TestDriver


class TestHysteresisDriver(TestDriver):
    def test_script(self):
        hd = oc.HysteresisDriver()

        script = hd._script(self.system, Hmin=(0, 0, 0),
                            Hmax=(10, 10, 10), n=10)

        assert script[0] == '#'
        assert script[-1] == '1'
        assert script.count('#') == 6
        assert script.count('Specify') == 4
        assert script.count('Destination') == 2
        assert script.count('Schedule') == 2
        assert script.count('mmArchive') == 2
        assert script.count('Stage') == 2

        assert 'Oxs_UZeeman' in script
        assert 'Oxs_CGEvolve' in script
        assert 'Oxs_MinDriver' in script
        assert 'Oxs_FileVectorField' in script

        if os.path.exists('tds'):
            shutil.rmtree('tds')

    @pytest.mark.oommf
    def test_drive(self):
        hd = oc.HysteresisDriver()

        hd.drive(self.system, Hmin=(0, 0, 0), Hmax=(1, 1, 1), n=10)

        dirname = os.path.join('tds', 'drive-{}'.format(self.system.drive_number-1))
        assert os.path.exists(dirname)
        miffilename = os.path.join(dirname, 'tds.mif')
        assert os.path.isfile(miffilename)

        omf_files = list(glob.iglob(os.path.join(dirname, '*.omf')))
        odt_files = list(glob.iglob(os.path.join(dirname, '*.odt')))

        assert len(omf_files) == 22
        omffilename = os.path.join(dirname, 'm0.omf')
        assert omffilename in omf_files

        assert len(odt_files) == 1

        if os.path.exists('tds'):
            shutil.rmtree('tds')
